@@ -5,19 +5,19 @@ struct Converter<QChar>
 {
     static bool checkType(PyObject* pyObj)
     {
-        return PyString_Check(pyObj) && (PyString_Size(pyObj) == 1);
+        return Shiboken::String::check(pyObj) && (Shiboken::String::len(pyObj) == 1);
     }
 
     static bool isConvertible(PyObject* pyObj)
     {
-        return (PyString_Check(pyObj) && (PyString_Size(pyObj) == 1))
+        return (Shiboken::String::check(pyObj) && (Shiboken::String::len(pyObj) == 1))
                || pyObj == Py_None
                || PyInt_Check(pyObj);
     }
 
     static QChar toCpp(PyObject* pyObj)
     {
-        if (PyString_Check(pyObj) && PyString_Size(pyObj) == 1)
+        if (Shiboken::String::check(pyObj) && (Shiboken::String::len(pyObj) == 1))
             return QChar(Shiboken::Converter<char >::toCpp(pyObj));
         else if (PyInt_Check(pyObj))
             return QChar(Shiboken::Converter<int >::toCpp(pyObj));
