@@ -26,6 +26,7 @@
 #include "pysideproperty_p.h"
 #include "pyside.h"
 #include "dynamicqmetaobject.h"
+#include "pysidemetafunction_p.h"
 
 #include <QHash>
 #include <QStringList>
@@ -34,6 +35,7 @@
 #include <gilstate.h>
 #include <QDebug>
 #include <limits>
+#include <algorithm>
 #include <typeresolver.h>
 #include <basewrapper.h>
 #include <conversions.h>
@@ -52,9 +54,8 @@ namespace {
     static PyObject *metaObjectAttr = 0;
 
     static int callMethod(QObject* object, int id, void** args);
-    static PyObject* parseArguments(QList<QByteArray> paramTypese, void** args);
+    static PyObject* parseArguments(const QList< QByteArray >& paramTypes, void** args);
     static bool emitShortCircuitSignal(QObject* source, int signalIndex, PyObject* args);
-    static bool emitNormalSignal(QObject* source, int signalIndex, const char* signal, PyObject* args, const QStringList& argTypes);
 
 #ifdef IS_PY3K
     static void destroyMetaObject(PyObject* obj)
@@ -338,13 +339,14 @@ bool SignalManager::emitSignal(QObject* source, const char* signal, PyObject* ar
 
     int signalIndex = source->metaObject()->indexOfSignal(signal);
     if (signalIndex != -1) {
-        bool isShortCircuit;
-        QStringList argTypes = Signal::getArgsFromSignature(signal, &isShortCircuit);
-
+        // cryptic but works!
+        // if the signature doesn't have a '(' it's a shor circuited signal, i.e. std::find
+        // returned the string null terminator.
+        bool isShortCircuit = !*std::find(signal, signal + std::strlen(signal), '(');
         if (isShortCircuit)
             return emitShortCircuitSignal(source, signalIndex, args);
         else
-            return emitNormalSignal(source, signalIndex, signal, args, argTypes);
+            return MetaFunction::call(source, signalIndex, args);
     }
     return false;
 }
@@ -418,28 +420,34 @@ int SignalManager::callPythonMetaMethod(const QMetaMethod& method, void** args,
     Q_ASSERT(pyMethod);
 
     Shiboken::GilState gil;
-    PyObject* pyArguments = NULL;
+    PyObject* pyArguments = 0;
 
     if (isShortCuit)
         pyArguments = reinterpret_cast<PyObject*>(args[1]);
     else
         pyArguments = parseArguments(method.parameterTypes(), args);
 
-    //keep the returnType this call be destroyed after method call
-    QByteArray returnType = method.typeName();
+    if (pyArguments) {
+        Shiboken::AutoDecRef retval(PyObject_CallObject(pyMethod, pyArguments));
 
-    Shiboken::AutoDecRef retval(PyObject_CallObject(pyMethod, pyArguments));
+        if (!isShortCuit && pyArguments)
+            Py_DECREF(pyArguments);
 
-    if (!isShortCuit)
-        Py_XDECREF(pyArguments);
-
-    if (retval.isNull()) {
-        PyErr_Print();
-    } else {
-        if (returnType.size() > 0)
-            Shiboken::TypeResolver::get(returnType)->toCpp(retval, &args[0]);
+        if (!retval.isNull() && retval != Py_None && !PyErr_Occurred()) {
+            const char* returnType = method.typeName();
+            if (returnType && std::strcmp("", returnType)) {
+                Shiboken::TypeResolver* typeResolver = Shiboken::TypeResolver::get(returnType);
+                if (typeResolver)
+                    typeResolver->toCpp(retval, &args[0]);
+                else
+                    PyErr_Format(PyExc_RuntimeError, "Can't fidn type resolver \"%s\" to call Python meta method.", returnType);
+            }
+        }
     }
 
+    if (PyErr_Occurred())
+        PyErr_Print();
+
     return -1;
 }
 
@@ -469,9 +477,9 @@ int SignalManager::registerMetaMethodGetIndex(QObject* source, const char* signa
             if (!dict || !PyDict_Contains(dict, metaObjectAttr)) {
                 dmo = new DynamicQMetaObject(pySelf->ob_type, metaObject);
 #ifdef IS_PY3K
-                PyObject *pyDmo = PyCapsule_New(dmo, 0, destroyMetaObject);
+                PyObject* pyDmo = PyCapsule_New(dmo, 0, destroyMetaObject);
 #else
-                PyObject *pyDmo = PyCObject_FromVoidPtr(dmo, destroyMetaObject);
+                PyObject* pyDmo = PyCObject_FromVoidPtr(dmo, destroyMetaObject);
 #endif
 
                 PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
@@ -539,15 +547,12 @@ static int callMethod(QObject* object, int id, void** args)
 }
 
 
-static PyObject* parseArguments(QList<QByteArray> paramTypes, void** args)
+static PyObject* parseArguments(const QList<QByteArray>& paramTypes, void** args)
 {
-    PyObject* preparedArgs = NULL;
-    Py_ssize_t argsSize = paramTypes.count();
+    int argsSize = paramTypes.count();
+    PyObject* preparedArgs = PyTuple_New(argsSize);
 
-    if (argsSize)
-        preparedArgs = PyTuple_New(argsSize);
-
-    for (int i = 0, max = paramTypes.count(); i < max; ++i) {
+    for (int i = 0, max = argsSize; i < max; ++i) {
         void* data = args[i+1];
         const char* dataType = paramTypes[i].constData();
 
@@ -556,12 +561,12 @@ static PyObject* parseArguments(QList<QByteArray> paramTypes, void** args)
             PyObject* arg = tr->toPython(data);
             PyTuple_SET_ITEM(preparedArgs, i, arg);
         } else {
+
             PyErr_Format(PyExc_TypeError, "Can't call meta function because I have no idea how to handle %s", dataType);
             Py_DECREF(preparedArgs);
-            return NULL;
+            return 0;
         }
     }
-
     return preparedArgs;
 }
 
@@ -572,49 +577,4 @@ static bool emitShortCircuitSignal(QObject* source, int signalIndex, PyObject* a
     return true;
 }
 
-static bool emitNormalSignal(QObject* source, int signalIndex, const char* signal, PyObject* args, const QStringList& argTypes)
-{
-    Shiboken::AutoDecRef sequence(PySequence_Fast(args, 0));
-    int argsGiven = PySequence_Fast_GET_SIZE(sequence.object());
-
-    if (argsGiven != argTypes.count()) {
-        PyErr_Format(PyExc_TypeError, "%s only accepts %d arguments, %d given!", signal, argTypes.count(), argsGiven);
-        return false;
-    }
-
-    QVariant* signalValues = new QVariant[argsGiven];
-    void** signalArgs = new void*[argsGiven + 1];
-    signalArgs[0] = 0;
-
-    int i;
-    for (i = 0; i < argsGiven; ++i) {
-        QByteArray typeName = argTypes[i].toAscii();
-        Shiboken::TypeResolver* typeResolver = Shiboken::TypeResolver::get(typeName);
-        if (typeResolver) {
-            if (Shiboken::TypeResolver::getType(typeName) == Shiboken::TypeResolver::ValueType) {
-                int typeId = QMetaType::type(typeName);
-                if (!typeId) {
-                    PyErr_Format(PyExc_TypeError, "Value type used on signal needs to be registered on meta type: %s", typeName.data());
-                    break;
-                }
-                signalValues[i] = QVariant(typeId, (void*) 0);
-            }
-            signalArgs[i+1] = signalValues[i].data();
-            typeResolver->toCpp(PySequence_Fast_GET_ITEM(sequence.object(), i), &signalArgs[i+1]);
-        } else {
-            PyErr_Format(PyExc_TypeError, "Unknown type used to emit a signal: %s", qPrintable(argTypes[i]));
-            break;
-        }
-    }
-
-    bool ok = i == argsGiven;
-    if (ok)
-        source->qt_metacall(QMetaObject::InvokeMetaMethod, signalIndex, signalArgs);
-
-    delete[] signalArgs;
-    delete[] signalValues;
-
-    return ok;
-}
-
 } //namespace
